# Generated by Django 2.1.13 on 2019-10-03 15:01

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import logging


def forward(apps, schema_editor):
    Scene = apps.get_model("terra_layer", "Scene")

    try:
        for slug, data in settings.TERRA_LAYER_VIEWS.items():
            Scene.objects.create(slug=slug, name=data["name"])
    except AttributeError:
        logging.info("No default layers' views was found. Don't forget to add some.")


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0040_filterfield_format_type")]

    operations = [
        migrations.CreateModel(
            name="Scene",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, unique=True)),
                ("slug", models.SlugField(max_length=255, unique=True)),
                ("category", models.CharField(default="map", max_length=255)),
                (
                    "custom_icon",
                    models.ImageField(
                        default=None, max_length=255, null=True, upload_to="scene-icons"
                    ),
                ),
            ],
            options={"permissions": (("can_manage_layers", "Can manage layers"),)},
        ),
        migrations.RunPython(forward, reverse_code=migrations.RunPython.noop),
        migrations.AlterField(
            model_name="layergroup",
            name="view",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="layer_groups",
                to="terra_layer.Scene",
            ),
        ),
    ]



# Generated by Django 2.2.8 on 2020-01-06 12:50

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import terra_layer.schema
import uuid


def create_uuid(apps, schema_editor):
    Layer = apps.get_model("terra_layer", "Layer")
    for inst in Layer.objects.all():
        inst.uuid = uuid.uuid4()
        inst.save()


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0046_migrate_layer_groups_2_tree"),
    ]

    operations = [
        migrations.AddField(
            model_name="layer",
            name="uuid",
            field=models.UUIDField(blank=True, null=True),
        ),
        migrations.RunPython(create_uuid, migrations.RunPython.noop),
        migrations.AlterField(
            model_name="layer",
            name="uuid",
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
        ),
        migrations.AlterField(
            model_name="scene",
            name="tree",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                default=list,
                validators=[
                    terra_layer.schema.JSONSchemaValidator(
                        limit_value={
                            "$id": "http://terralego.com/scene_layertree.json",
                            "$schema": "http://json-schema.org/draft-07/schema#",
                            "definitions": {},
                            "items": {
                                "$id": "#/items",
                                "dependencies": {"group": ["children", "label"]},
                                "properties": {
                                    "children": {"$ref": "#"},
                                    "expanded": {
                                        "$id": "#/items/properties/expanded",
                                        "default": False,
                                        "examples": [True],
                                        "title": "The expanded status in admin. Not used yet",
                                        "type": "boolean",
                                    },
                                    "geolayer": {
                                        "$id": "#/items/properties/geolayer",
                                        "default": 0,
                                        "examples": [96],
                                        "title": "The geolayer id",
                                        "type": "integer",
                                    },
                                    "group": {
                                        "$id": "#/items/properties/exclusive",
                                        "default": False,
                                        "examples": [True],
                                        "title": "Is the group exclusive ?",
                                        "type": "boolean",
                                    },
                                    "label": {
                                        "$id": "#/items/properties/label",
                                        "default": "",
                                        "examples": ["My Group"],
                                        "pattern": "^(.*)$",
                                        "title": "The group name",
                                        "type": "string",
                                    },
                                    "selectors": {
                                        "$id": "#/items/properties/selectors",
                                        "title": "The selectors for this group",
                                        "type": ["array", "null"],
                                    },
                                    "settings": {
                                        "$id": "#/items/properties/settings",
                                        "title": "The settings of group",
                                        "type": "object",
                                    },
                                },
                                "required": [],
                                "title": "Layer tree item",
                                "type": "object",
                            },
                            "title": "Scene layer tree schema",
                            "type": "array",
                        }
                    )
                ],
            ),
        ),
    ]



# Generated by Django 2.1.9 on 2019-07-18 15:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0031_auto_20190718_1601")]

    operations = [
        migrations.AlterUniqueTogether(
            name="layergroup", unique_together={("view", "label", "parent")}
        )
    ]



# Generated by Django 2.1.9 on 2019-06-12 08:20

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0008_auto_20190611_1323")]

    operations = [
        migrations.AlterField(
            model_name="filterfield",
            name="filter_settings",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        )
    ]



from django.urls import path, include
from rest_framework import routers

from ..views import (
    GeostoreLayerViewSet,
    GeostoreLayerGroupViewsSet,
    GeostoreFeatureViewSet,
)


router = routers.DefaultRouter()

router.register(r"layer", GeostoreLayerViewSet, basename="layer")
router.register(r"group", GeostoreLayerGroupViewsSet, basename="group"),

router.register(
    r"layer/(?P<layer>[\d\w\-_]+)/feature", GeostoreFeatureViewSet, basename="feature"
)

urlpatterns = [path("", include(router.urls))]



# Generated by Django 2.1.11 on 2019-09-04 14:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0037_auto_20190808_1814")]

    operations = [
        migrations.AddField(
            model_name="layer",
            name="highlight_color",
            field=models.CharField(blank=True, max_length=255),
        )
    ]



# Generated by Django 2.0.13 on 2019-05-24 09:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0001_initial")]

    operations = [
        migrations.AlterField(
            model_name="filterfield",
            name="field",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="filter_field",
                to="django_geosource.Field",
            ),
        ),
        migrations.AlterField(
            model_name="filterfield",
            name="filter_type",
            field=models.IntegerField(default=0),
        ),
    ]



# Generated by Django 2.1.9 on 2019-07-18 08:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0029_auto_20190716_1643")]

    operations = [
        migrations.AddField(
            model_name="layergroup", name="order", field=models.IntegerField(default=0)
        )
    ]



# Generated by Django 2.0.13 on 2019-06-20 12:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0012_auto_20190618_1500")]

    operations = [
        migrations.RemoveField(model_name="layer", name="filter_enable"),
        migrations.AddField(
            model_name="filterfield",
            name="filter_enable",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.0.13 on 2019-06-06 14:35

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0004_auto_20190605_1638")]

    operations = [
        migrations.AlterField(
            model_name="layer",
            name="layer_style",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True, default={}
            ),
        )
    ]



"""
All json schema for validation
"""

import django
from django.core.validators import BaseValidator
import jsonschema


class JSONSchemaValidator(BaseValidator):
    def compare(self, a, b):
        try:
            jsonschema.validate(a, b)
        except jsonschema.exceptions.ValidationError:
            raise django.core.exceptions.ValidationError(
                "%(value)s failed JSON schema check", params={"value": a}
            )


SCENE_LAYERTREE = {
    "definitions": {},
    "$schema": "http://json-schema.org/draft-07/schema#",
    "$id": "http://terralego.com/scene_layertree.json",
    "type": "array",
    "title": "Scene layer tree schema",
    "items": {
        "$id": "#/items",
        "type": "object",
        "title": "Layer tree item",
        "required": [],
        "dependencies": {"group": ["children", "label"]},
        "properties": {
            "label": {
                "$id": "#/items/properties/label",
                "type": "string",
                "title": "The group name",
                "default": "",
                "examples": ["My Group"],
                "pattern": "^(.*)$",
            },
            "expanded": {
                "$id": "#/items/properties/expanded",
                "type": "boolean",
                "title": "The expanded status in admin. Not used yet",
                "default": False,
                "examples": [True],
            },
            "geolayer": {
                "$id": "#/items/properties/geolayer",
                "type": "integer",
                "title": "The geolayer id",
                "default": 0,
                "examples": [96],
            },
            "group": {
                "$id": "#/items/properties/group",
                "type": "boolean",
                "title": "The group name. Present if it's a group.",
                "default": False,
                "examples": [True],
            },
            "selectors": {
                "$id": "#/items/properties/selectors",
                "type": ["array", "null"],
                "title": "The selectors for this group",
            },
            "settings": {
                "$id": "#/items/properties/settings",
                "type": "object",
                "title": "The settings of group",
            },
            "children": {"$ref": "#"},
        },
    },
}



# Generated by Django 2.0.13 on 2019-07-04 08:12

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0023_layer_interactions")]

    operations = [
        migrations.AddField(
            model_name="customstyle",
            name="interactions",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=list),
        )
    ]



# Generated by Django 2.1.9 on 2019-07-15 14:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0026_layer_group")]

    operations = [
        migrations.RemoveField(model_name="layer", name="view"),
        migrations.AlterField(
            model_name="layer",
            name="group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="terra_layer.LayerGroup",
            ),
        ),
    ]



# Generated by Django 2.2.7 on 2019-12-12 10:29

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import terra_layer.schema


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0044_order"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="layer", options={"ordering": ("order", "name")},
        ),
        migrations.AddField(
            model_name="scene",
            name="tree",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                default=list,
                validators=[
                    terra_layer.schema.JSONSchemaValidator(
                        limit_value={
                            "$id": "http://terralego.com/scene_layertree.json",
                            "$schema": "http://json-schema.org/draft-07/schema#",
                            "definitions": {},
                            "items": {
                                "$id": "#/items",
                                "dependencies": {"group": ["children", "label"]},
                                "properties": {
                                    "children": {"$ref": "#"},
                                    "expanded": {
                                        "$id": "#/items/properties/expanded",
                                        "default": False,
                                        "examples": [True],
                                        "title": "The expanded status in admin. Not used yet",
                                        "type": "boolean",
                                    },
                                    "geolayer": {
                                        "$id": "#/items/properties/geolayer",
                                        "default": 0,
                                        "examples": [96],
                                        "title": "The geolayer id",
                                        "type": "integer",
                                    },
                                    "group": {
                                        "$id": "#/items/properties/group",
                                        "default": False,
                                        "examples": [True],
                                        "title": "The group name. Present if it's a group.",
                                        "type": "boolean",
                                    },
                                    "label": {
                                        "$id": "#/items/properties/label",
                                        "default": "",
                                        "examples": ["My Group"],
                                        "pattern": "^(.*)$",
                                        "title": "The group name",
                                        "type": "string",
                                    },
                                },
                                "required": [],
                                "title": "Layer tree item",
                                "type": "object",
                            },
                            "title": "Scene layer tree schema",
                            "type": "array",
                        }
                    )
                ],
            ),
        ),
        migrations.AlterField(
            model_name="layer",
            name="group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="layers",
                to="terra_layer.LayerGroup",
            ),
        ),
        migrations.AlterUniqueTogether(name="layergroup", unique_together=set(),),
    ]



# Generated by Django 2.2.5 on 2019-11-05 03:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0041_auto_20191003_1701")]

    operations = [
        migrations.AlterModelOptions(name="layer", options={"ordering": ("order",)}),
        migrations.AlterModelOptions(name="scene", options={}),
    ]



# Generated by Django 2.2.7 on 2019-11-19 07:17

from django.db import migrations, models
from django.db.models import F


def forward(apps, schema_editor):
    Scene = apps.get_model("terra_layer", "Scene")
    Scene.objects.update(order=F("pk"))


class Migration(migrations.Migration):
    dependencies = [
        ("terra_layer", "0043_layer_active_by_default"),
    ]

    operations = [
        migrations.AlterModelOptions(name="scene", options={"ordering": ["order"]}),
        migrations.AddField(
            model_name="scene",
            name="order",
            field=models.IntegerField(db_index=True, default=0),
        ),
        migrations.RunPython(forward, migrations.RunPython.noop),
    ]



# Generated by Django 2.2.5 on 2019-11-12 04:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("terra_layer", "0042_auto_20191105_0307"),
    ]

    operations = [
        migrations.AddField(
            model_name="layer",
            name="active_by_default",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 2.1.9 on 2019-07-16 10:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0027_auto_20190715_1631")]

    operations = [
        migrations.AddField(
            model_name="layergroup",
            name="exclusive",
            field=models.BooleanField(default=False),
        )
    ]



import argparse
from django.core.management.base import BaseCommand, CommandError
from terra_layer.models import Layer, Scene
from terra_layer.serializers import LayerDetailSerializer
from django_geosource.models import Source
import json


class Command(BaseCommand):
    help = "Load a dumped layer"

    def add_arguments(self, parser):
        parser.add_argument(
            "-file",
            type=argparse.FileType("r"),
            required=True,
            action="store",
            help="json file path",
        )

    def handle(self, *args, **options):
        data = json.load(options["file"])

        source = Source.objects.get(slug=data["source"])

        fk_fields = (
            (Scene, "view", "slug"),
            (Source, "source", "slug"),
        )

        for klass, field, sfield in fk_fields:
            if data.get(field):
                data[field] = klass.objects.get(**{sfield: data[field]}).pk

        for cs in data["custom_styles"]:
            cs["source"] = Source.objects.get(slug=cs["source"]).pk

        if data.get("main_field"):
            data["main_field"] = source.fields.get(name=data["main_field"]).pk

        for field in data["fields"]:
            field["field"] = source.fields.get(name=field["field"]).pk

        parts = data["name"].split("/")
        layer_name = parts.pop()

        # Try to find already existing layer
        try:
            layer = Layer.objects.get(uuid=data["uuid"])
            exists = True
        except Exception:
            layer = Layer()
            exists = False

        del data["group"]  # Remove group as we compute it later
        data["name"] = layer_name

        layer_detail_serializer = LayerDetailSerializer(instance=layer, data=data)
        try:
            layer_detail_serializer.is_valid(raise_exception=True)
        except Exception as e:
            raise CommandError(f"A validation error occurred with data: {e}")

        layer_detail_serializer.save()

        # Here we insert layer in tree if not previously existing
        if not exists:
            scene = Scene.objects.get(id=data["view"])
            scene.insert_in_tree(layer_detail_serializer.instance, parts)



# Generated by Django 2.1.9 on 2019-07-23 12:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0034_layer_in_tree")]

    operations = [
        migrations.AlterField(
            model_name="layer",
            name="popup_maxzoom",
            field=models.FloatField(default=22),
        ),
        migrations.AlterField(
            model_name="layer", name="popup_minzoom", field=models.FloatField(default=0)
        ),
    ]



# Generated by Django 2.0.13 on 2019-05-28 14:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0002_auto_20190524_1151")]

    operations = [
        migrations.AlterField(
            model_name="filterfield",
            name="field",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="django_geosource.Field"
            ),
        ),
        migrations.AlterField(
            model_name="filterfield",
            name="layer",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="fields_filters",
                to="terra_layer.Layer",
            ),
        ),
    ]



# Generated by Django 2.0.13 on 2019-06-12 15:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0009_auto_20190612_1020")]

    operations = [
        migrations.AlterField(
            model_name="filterfield",
            name="filter_type",
            field=models.CharField(max_length=255, null=True),
        )
    ]



# Generated by Django 2.1.11 on 2019-08-08 16:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0036_auto_20190731_1848")]

    operations = [
        migrations.AlterModelOptions(
            name="filterfield", options={"ordering": ("order",)}
        ),
        migrations.AddField(
            model_name="filterfield", name="order", field=models.IntegerField(default=0)
        ),
    ]



import tempfile

from django.core.management import call_command, get_commands
from django.conf import settings
from django.core.cache import cache
from django.db.models import Prefetch, Q
from django.http import Http404, QueryDict
from django.urls import reverse
from django.utils.functional import cached_property
from django.utils.http import urlunquote
from django_geosource.models import WMTSSource

from geostore.tokens import tiles_token_generator

from rest_framework.generics import get_object_or_404
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.viewsets import ModelViewSet
from rest_framework.serializers import ValidationError

from ..models import Layer, LayerGroup, FilterField, Scene
from ..permissions import LayerPermission, ScenePermission
from ..serializers import (
    LayerListSerializer,
    LayerDetailSerializer,
    SceneListSerializer,
    SceneDetailSerializer,
)
from ..sources_serializers import SourceSerializer
from ..utils import dict_merge, get_layer_group_cache_key


class SceneViewset(ModelViewSet):
    model = Scene
    queryset = Scene.objects.all()
    permission_classes = (ScenePermission,)

    def get_serializer_class(self,):
        if self.action in ["retrieve", "update", "create", "partial_update"]:
            return SceneDetailSerializer
        return SceneListSerializer

    def check_layer_status(self, view_id, current_node):
        """
        Check all layers in tree to valide existence and scene ownership.
        Recursive process.

        :param current_node: Current node from the tree
        """

        for item in current_node:
            if "geolayer" in item:
                try:
                    # Is layer deleted ?
                    layer = Layer.objects.get(pk=item["geolayer"])
                except Layer.DoesNotExist:
                    raise ValidationError(
                        f"Layer {item['geolayer']} doesn't exists anymore"
                    )

                # Is layer owned by another scene ?
                if layer.group and layer.group.view.id != view_id:
                    raise ValidationError(
                        f"Layer {item['geolayer']} can't be stolen from another scene"
                    )
            else:
                # And we start with the new node
                self.check_layer_status(view_id, item["children"])

    def handle_import_file(self, scene_name):
        if "load_xls" in get_commands() and "file" in self.request.FILES:
            # Handle imported file
            with tempfile.NamedTemporaryFile(suffix=".xls") as xls_file:
                for chunk in self.request.FILES["file"]:
                    xls_file.write(chunk)

                xls_file.seek(0)

                call_command("load_xls", scene_name=scene_name, file=xls_file.name)

    def perform_update(self, serializer):

        if serializer.is_valid():
            self.check_layer_status(
                serializer.instance.id, serializer.validated_data.get("tree", [])
            )
            serializer.save()

        self.handle_import_file(serializer.instance.name)

    def perform_create(self, serializer):
        if serializer.is_valid():
            self.check_layer_status(None, serializer.validated_data.get("tree", []))
            serializer.save()

        self.handle_import_file(serializer.instance.name)


class LayerViewset(ModelViewSet):
    model = Layer
    ordering_fields = (
        "name",
        "source__name",
        "group__view__name",
        "active_by_default",
        "order",
        "in_tree",
    )
    filter_fields = (
        "source",
        "group",
        "active_by_default",
        "in_tree",
        "table_enable",
        "popup_enable",
        "minisheet_enable",
    )
    permission_classes = (LayerPermission,)
    search_fields = ["name", "settings"]

    def get_queryset(self):
        return self.model.objects.all()

    def get_serializer_class(self,):
        if self.action in ["retrieve", "update", "create", "partial_update"]:
            return LayerDetailSerializer
        return LayerListSerializer

    def perform_destroy(self, instance):
        if instance.group:  #  Prevent deletion of layer used in any layer tree
            raise ValidationError("Can't delete a layer linked to a scene")
        super().perform_destroy(instance)


class LayerView(APIView):
    """ This view generates the LayersTree used to construct the frontend
    """

    permission_classes = ()
    model = Layer
    EXTERNAL_SOURCES_CLASSES = [WMTSSource]
    DEFAULT_SOURCE_NAME = "terra"
    DEFAULT_SOURCE_TYPE = "vector"

    prefetch_layers = Prefetch(
        "layers",
        (
            Layer.objects.select_related("source").prefetch_related(
                Prefetch(
                    "fields_filters",
                    FilterField.objects.filter(shown=True).select_related("field"),
                    to_attr="filters_shown",
                ),
                Prefetch(
                    "fields_filters",
                    FilterField.objects.filter(filter_enable=True).select_related(
                        "field"
                    ),
                    to_attr="filters_enabled",
                ),
                "custom_styles__source",
            )
        ),
    )

    scene = None

    def get(self, request, slug=None, format=None):
        self.scene = get_object_or_404(Scene, slug=slug)
        self.layergroup = self.layers.first().source.get_layer().layer_groups.first()

        self.user_groups = tiles_token_generator.get_groups_intersect(
            self.request.user, self.layergroup
        )

        cache_key = get_layer_group_cache_key(
            self.scene, self.user_groups.values_list("name", flat=True)
        )

        response = cache.get_or_set(cache_key, self.get_response_with_sources)

        return Response(response)

    def get_response_with_sources(self):
        """ Return a response object containing the full layersTree with updated
        user authentication.
        """

        layer_structure = self.get_layer_structure()

        tilejson_url = reverse("group-tilejson", args=(self.layergroup.slug,))
        querystring = QueryDict(mutable=True)

        # When the user is not anonymous, we provide tokens in the URL to authenticated
        # it in the MVT endpoint
        if not self.request.user.is_anonymous:
            querystring.update(
                {
                    "idb64": tiles_token_generator.token_idb64(
                        self.user_groups, self.layergroup
                    ),
                    "token": tiles_token_generator.make_token(
                        self.user_groups, self.layergroup
                    ),
                }
            )

        layer_structure["map"]["customStyle"]["sources"] = [
            {
                "id": self.DEFAULT_SOURCE_NAME,
                "type": self.DEFAULT_SOURCE_TYPE,
                "url": f"{tilejson_url}?{querystring.urlencode()}",
            }
        ]
        return layer_structure

    def get_layer_structure(self):
        """ Return the structured layerTree
        """
        return {
            "title": self.scene.name,
            "type": self.scene.category,
            "layersTree": self.get_layers_tree(self.scene),
            "interactions": self.get_interactions(self.layers),
            "map": {
                **settings.TERRA_DEFAULT_MAP_SETTINGS,
                "customStyle": {"sources": [], "layers": self.get_map_layers()},
            },
        }

    def get_map_layers(self):
        """ Return sources informations using serializer from sources_serializers module
        """
        map_layers = []
        for layer in self.layers.filter(source__slug__in=self.authorized_sources):
            map_layers += [
                SourceSerializer.get_object_serializer(layer).data,
                *[
                    SourceSerializer.get_object_serializer(cs).data
                    for cs in layer.custom_styles.filter(
                        source__slug__in=self.authorized_sources
                    )
                ],
            ]
        return map_layers

    def get_interactions(self, layers):
        """ Return interactions for all layers in the scene
        """
        interactions = []
        for layer in layers:
            interactions += self.get_interactions_for_layer(layer)
        return interactions

    def get_formatted_interactions(self, layer):
        """ Return all interactions of a layer after beeing formatted correctly
        for the frontend
        """
        return [
            {
                "id": layer.layer_identifier,
                "fetchProperties": {
                    "url": urlunquote(
                        reverse(
                            "feature-detail",
                            args=(layer.source.get_layer().pk, "{{id}}"),
                        )
                    ),
                    "id": "_id",
                },
                **interaction,
            }
            for interaction in layer.interactions
        ]

    def get_interactions_for_layer(self, layer):
        """ Return formatted interaction of a layer

        It contains, popup, minisheet and custom styles
        """
        interactions = self.get_formatted_interactions(layer)
        for cs in layer.custom_styles.all():
            interactions += self.get_formatted_interactions(cs)

        if layer.popup_enable:
            interactions.append(
                {
                    "id": layer.layer_identifier,
                    "interaction": "displayTooltip",
                    "trigger": "mouseover",
                    "template": layer.popup_template,
                    "constraints": [
                        {"minZoom": layer.popup_minzoom, "maxZoom": layer.popup_maxzoom}
                    ],
                }
            )

        if layer.minisheet_enable:
            settings_interactions = {
                "id": layer.layer_identifier,
                "interaction": "displayDetails",
                "template": layer.minisheet_template,
                "fetchProperties": {
                    "url": urlunquote(
                        reverse(
                            "feature-detail",
                            args=(layer.source.get_layer().pk, "{{id}}"),
                        )
                    ),
                    "id": "_id",
                },
            }
            if layer.highlight_color:
                settings_interactions["highlight_color"] = layer.highlight_color

            interactions.append(settings_interactions)

        return interactions

    def get_layers_list_for_layer(self, layer):
        """ Return list of sublayers of a layer
        """
        return [
            layer.layer_identifier,
            *[s.layer_identifier for s in layer.custom_styles.all()],
        ]

    def get_layers_tree(self, scene):
        """ Return the full layer tree of a scene object
        """
        root_group = LayerGroup.objects.prefetch_related(self.prefetch_layers).get(
            view=scene, parent=None
        )

        # Keep only child of root group
        return self.get_group_dict(root_group)["layers"]

    def get_group_dict(self, group):
        """ Recursive method that return the tree from a LayerGroup element.

        `group.settings` is injected in the group dictionnary, so any setting can be overridden.

        """
        group_content = {
            "group": group.label,
            "exclusive": group.exclusive,
            "selectors": group.selectors,
            "order": group.order,
            "layers": [],
            **group.settings,
        }

        # Add subgroups
        for sub_group in group.children.filter(view=group.view).prefetch_related(
            self.prefetch_layers
        ):
            group_dict = self.get_group_dict(sub_group)
            # exclude empty groups
            if group_dict["layers"]:
                group_content["layers"].append(group_dict)

        # Add layers of group
        for layer in group.layers.filter(in_tree=True):
            layer_dict = self.get_layer_dict(layer)
            if layer_dict:
                group_content["layers"].append(layer_dict)

        # Group en layer ordering
        group_content["layers"].sort(key=lambda x: x["order"])

        # Remove key order as not part of schema
        [item.pop("order") for item in group_content["layers"]]

        return group_content

    def get_layer_dict(self, layer):
        if (
            layer.source.slug not in self.authorized_sources
            or layer.custom_styles.exclude(
                source__slug__in=self.authorized_sources
            ).exists()
        ):
            # Exclude layers with non-authorized sources
            return None

        default_values = {
            "initialState": {
                "active": layer.active_by_default,
                "opacity": layer.settings.get("default_opacity", 100) / 100,
            }
        }

        main_field = getattr(layer.main_field, "name", None)

        # Construct the layer object
        layer_object = {
            **dict_merge(default_values, layer.settings),
            "label": layer.name,
            "order": layer.order,
            "content": layer.description,
            "layers": self.get_layers_list_for_layer(layer),
            "legends": layer.legends,
            "mainField": main_field,
            "filters": {
                "layer": layer.source.slug,
                "mainField": main_field,
                "fields": self.get_filter_fields_for_layer(layer),
                "form": self.get_filter_forms_for_layer(layer),
            },
        }

        # Set the exportable status of the layer if any filter fields is exportable
        layer_object["filters"]["exportable"] = any(
            [f["exportable"] for f in layer_object["filters"]["fields"] or []]
        )

        return layer_object

    def get_filter_fields_for_layer(self, layer):
        """ Return the filter fields of the layer if table is enabled
        """
        if layer.table_enable:
            return [
                {
                    "value": field_filter.field.name,
                    "label": field_filter.label or field_filter.field.label,
                    "exportable": field_filter.exportable,
                    "format_type": field_filter.format_type,
                    "display": field_filter.display,
                    "settings": field_filter.settings,
                }
                for field_filter in layer.filters_shown
            ]

    def get_filter_forms_for_layer(self, layer):
        """ Return forms of a layer if filters are enabled
        """
        if layer.filters_enabled:
            return [
                {
                    "property": field_filter.field.name,
                    "label": field_filter.label or field_filter.field.label,
                    **field_filter.filter_settings,
                }
                for field_filter in layer.filters_enabled
            ]

    @cached_property
    def authorized_sources(self):
        """ Cached property of authorized sources from the authenticated user's groups
        """
        groups = self.user_groups
        sources_slug = list(
            self.layergroup.layers.filter(
                Q(authorized_groups__isnull=True) | Q(authorized_groups__in=groups)
            ).values_list("name", flat=True)
        ) + list(WMTSSource.objects.values_list("slug", flat=True))

        return sources_slug

    @cached_property
    def layers(self):
        """ List of layers of the selected scene
        """
        layers = (
            self.model.objects.filter(group__view=self.scene.pk)
            .order_by("order")
            .select_related("source")
            .prefetch_related("custom_styles__source")
        )

        if layers:
            return layers
        raise Http404



# Generated by Django 2.1.10 on 2019-07-31 16:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0035_auto_20190723_1418")]

    operations = [
        migrations.AlterModelOptions(
            name="layer",
            options={
                "ordering": ("order",),
                "permissions": (("can_manage_layers", "Can manage layers"),),
            },
        ),
        migrations.AlterField(
            model_name="layergroup",
            name="parent",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="children",
                to="terra_layer.LayerGroup",
            ),
        ),
    ]



# Generated by Django 2.0.13 on 2019-06-11 09:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0006_auto_20190607_1151")]

    operations = [
        migrations.AlterField(
            model_name="layer",
            name="view",
            field=models.IntegerField(
                choices=[
                    (1, "Visualiser"),
                    (2, "Analyser"),
                    (3, "Découvrir les espaces d'activité"),
                ]
            ),
        )
    ]



from django.core.management.base import BaseCommand, CommandError
from terra_layer.models import Layer, LayerGroup, Scene
from terra_layer.serializers import LayerDetailSerializer
from django_geosource.models import Field, Source
import json


class Command(BaseCommand):
    help = "Dump a layer to json format"

    def add_arguments(self, parser):
        parser.add_argument(
            "-pk", type=int, action="store", help="Pk of the layer to export"
        )

    def handle(self, *args, **options):
        try:
            self.layer = Layer.objects.get(pk=options.get("pk"))
        except Layer.DoesNotExist:
            raise CommandError("Layer does not exist")

        serialized = LayerDetailSerializer(self.layer).data
        self.clean_ids(serialized)

        self.stdout.write(json.dumps(serialized))

    def clean_ids(self, serialized):
        excluded_fields = ("id",)
        for field in excluded_fields:
            serialized.pop(field)

        # Clean custom_style id
        for cs in serialized.get("custom_styles", []):
            cs.pop("id")
            cs["source"] = Source.objects.get(pk=cs["source"]).slug

        for field in serialized.get("fields", []):
            field.pop("id")
            field.pop("sourceFieldId")
            field["field"] = Field.objects.get(pk=field["field"]).name

        fk_fields = (
            (LayerGroup, "group", "label"),
            (Scene, "view", "slug"),
            (Source, "source", "slug"),
            (Field, "main_field", "name"),
        )

        for klass, field, sfield in fk_fields:
            if serialized.get(field):
                serialized[field] = getattr(
                    klass.objects.get(pk=serialized.get(field)), sfield
                )



from .layers import SceneViewset, LayerViewset, LayerView  # NOQA
from .extras import (  # NOQA
    GeoSourceModelViewset,
    GeostoreLayerViewSet,
    GeostoreFeatureViewSet,
    GeostoreLayerGroupViewsSet,
)



from django.urls import include, path
from rest_framework import routers

from .geostore import urlpatterns as geostore_patterns
from .geosource import router as geosource_router
from ..views import LayerViewset, LayerView, SceneViewset

router = routers.SimpleRouter()

router.register(r"geolayer/scene", SceneViewset, basename="scene")
router.register(r"geolayer", LayerViewset, basename="layer")

# Extras viewsets

urlpatterns = [
    path(r"geolayer/view/<str:slug>/", LayerView.as_view(), name="layerview"),
    # Extra urls from third part modules
    path("", include("terra_accounts.urls")),
    path("", include("terra_utils.urls")),
    path("geostore/", include(geostore_patterns)),
    path(
        "geosource/",
        include((geosource_router.urls, "geosource"), namespace="geosource"),
    ),
]

urlpatterns += router.urls



# Generated by Django 2.1.12 on 2019-09-24 09:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0039_layer_main_field")]

    operations = [
        migrations.AddField(
            model_name="filterfield",
            name="format_type",
            field=models.CharField(default=None, max_length=255, null=True),
        )
    ]



# Generated by Django 2.0.13 on 2019-06-05 14:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("terra_layer", "0003_auto_20190528_1629")]

    operations = [
        migrations.RenameField(
            model_name="layer", old_name="filter_fields", new_name="fields"
        ),
        migrations.RemoveField(model_name="layer", name="table_export_fields"),
        migrations.RemoveField(model_name="layer", name="table_fields"),
        migrations.AddField(
            model_name="filterfield",
            name="exportable",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="filterfield",
            name="shown",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="layer",
            name="popup_maxzoom",
            field=models.FloatField(default=10),
        ),
        migrations.AddField(
            model_name="layer",
            name="popup_minzoom",
            field=models.FloatField(default=10),
        ),
    ]
